@@ -1,51 +1,120 @@
-#if OCAML_VERSION < (4, 03, 0)
-#define Pconst_char Const_char
-#define Pconst_string Const_string
-#define Pstr_type(rec_flag, type_decls) Pstr_type(type_decls)
-#define Psig_type(rec_flag, type_decls) Psig_type(type_decls)
-#endif
+open Ppxlib
 
-#if OCAML_VERSION < (4, 08, 0)
-#define Attribute_expr(loc_, txt_, payload) ({txt = txt_; loc = loc_}, payload)
-#define Attribute_patt(loc_, txt_, payload) ({txt = txt_; loc = loc_}, payload)
-#else
-#define Attribute_expr(loc_, txt_, payload) { attr_name = \
-                                                { txt = txt_; loc = loc_ }; \
-                                              attr_payload = payload; \
-                                              attr_loc = loc_ }
-#define Attribute_patt(loc_, txt_, payload) { attr_name = \
-                                               { txt = txt_; loc = loc_ }; \
-                                              attr_payload = payload; \
-                                              attr_loc = _ }
-#endif
-
-#if OCAML_VERSION < (4, 08, 0)
-#define Rtag_patt(label, constant, args) Rtag(label, _, constant, args)
-#define Rinherit_patt(typ) Rinherit(typ)
-#else
-#define Rtag_patt(label, constant, args) {prf_desc = Rtag(label, constant, args); _}
-#define Rinherit_patt(typ) {prf_desc = Rinherit(typ); _}
-#endif
-
-#if OCAML_VERSION < (4, 11, 0)
-#define Pconst_string_patt(s, loc) Pconst_string (s, loc)
-#else
-#define Pconst_string_patt(s, loc) Pconst_string (s, loc, _)
-#endif
-
-open Longident
-open Location
 open Asttypes
-open Parsetree
 open Ast_helper
-open Ast_convenience
-open Ppx_deriving_runtime
 
-#if OCAML_VERSION >= (4, 05, 0)
+module Ast_convenience = struct
+  (* Formerly defined in Ppx_tools.Ast_convenience.
+     Ppx_tools is not compatible with Ppxlib. *)
+
+  let mkloc txt loc =
+    { txt; loc }
+
+  let mknoloc txt =
+    mkloc txt !Ast_helper.default_loc
+
+  let str_of_string s =
+    mknoloc s
+
+  let lid_of_string s =
+    mknoloc (Longident.parse s)
+
+  let unit () =
+    let loc = !Ast_helper.default_loc in
+    [%expr ()]
+
+  let punit () =
+    let loc = !Ast_helper.default_loc in
+    [%pat? ()]
+
+  let str s =
+    Ast_helper.Exp.constant (Ast_helper.Const.string s)
+
+  let int i =
+    Ast_helper.Exp.constant (Ast_helper.Const.int i)
+
+  let pint i =
+    Ast_helper.Pat.constant (Ast_helper.Const.int i)
+
+  let evar name =
+    Ast_helper.Exp.ident (lid_of_string name)
+
+  let pvar name =
+    Ast_helper.Pat.var (str_of_string name)
+
+  let app f args =
+    match args with
+    | [] -> f
+    | _ ->
+        let args = List.map (fun e -> (Nolabel, e)) args in
+        Ast_helper.Exp.apply f args
+
+  let constr name args =
+    let args =
+      match args with
+      | [] -> None
+      | [arg] -> Some arg
+      | _ -> Some (Ast_helper.Exp.tuple args) in
+    Ast_helper.Exp.construct (lid_of_string name) args
+
+  let pconstr name args =
+    let args =
+      match args with
+      | [] -> None
+      | [arg] -> Some arg
+      | _ -> Some (Ast_helper.Pat.tuple args) in
+    Ast_helper.Pat.construct (lid_of_string name) args
+
+  let tconstr name args =
+    Ast_helper.Typ.constr (lid_of_string name) args
+
+  let record fields =
+    let fields =
+      List.map (fun (name, value) -> (lid_of_string name, value)) fields in
+    Ast_helper.Exp.record fields None
+
+  let precord ~closed fields =
+    let fields =
+      List.map (fun (name, value) -> (lid_of_string name, value)) fields in
+    Ast_helper.Pat.record fields closed
+
+  let tuple items =
+    match items with
+    | [] -> unit ()
+    | [item] -> item
+    | _ -> Ast_helper.Exp.tuple items
+
+  let ptuple items =
+    match items with
+    | [] -> punit ()
+    | [item] -> item
+    | _ -> Ast_helper.Pat.tuple items
+
+  let attribute_has_name name attribute =
+    attribute.attr_name.txt = name
+
+  let has_attr name attributes =
+    List.exists (attribute_has_name name) attributes
+
+  let find_attr name attributes =
+    match List.find (attribute_has_name name) attributes with
+    | exception Not_found -> None
+    | attribute -> Some attribute.attr_payload
+
+  module Label = struct
+    let nolabel = Nolabel
+
+    let labelled s =
+      Labelled s
+
+    let optional s =
+      Optional s
+  end
+end
+
+open Ast_convenience
+
 type tyvar = string Location.loc
-#else
-type tyvar = string
-#endif
 
 type deriver = {
   name : string ;
@@ -100,6 +169,7 @@ let lookup name =
   | Some (External _) | None -> None
 
 let raise_errorf ?sub ?loc fmt =
+  let module Location = Ocaml_common.Location in
   let raise_msg str =
 #if OCAML_VERSION >= (4, 08, 0)
     let sub =
@@ -110,7 +180,7 @@ let raise_errorf ?sub ?loc fmt =
 #endif
     let err = Location.error ?sub ?loc str in
     raise (Location.Error err) in
-  Printf.kprintf raise_msg fmt
+  Printf.ksprintf raise_msg fmt
 
 let create =
   let def_ext_str name ~options ~path typ_ext =
@@ -149,16 +219,10 @@ let create =
 let string_of_core_type typ =
   Format.asprintf "%a" Pprintast.core_type { typ with ptyp_attributes = [] }
 
-type constant =
-  #if OCAML_VERSION >= (4, 03, 0)
-    Parsetree.constant
-  #else
-    Asttypes.constant
-  #endif
-
-let string_of_constant_opt (constant : constant) : string option =
+let string_of_constant_opt (constant : Parsetree.constant) : string option =
   match constant with
-  | Pconst_string_patt(s, _) -> Some s
+  | Pconst_string (s, _, _) ->
+      Some s
   | _ -> None
 
 let string_of_expression_opt (e : Parsetree.expression) : string option =
@@ -167,34 +231,14 @@ let string_of_expression_opt (e : Parsetree.expression) : string option =
       string_of_constant_opt constant
   | _ -> None
 
-#if OCAML_VERSION >= (4, 03, 0)
-  module Const = Ast_helper.Const
-#else
-  module Const = struct
-    let integer ?suffix:_ i = Const_int (int_of_string i)
-    let int ?suffix:_ i = Const_int i
-    let int32 ?suffix:_ i = Const_int (Int32.to_int i)
-    let int64 ?suffix:_ i = Const_int (Int64.to_int i)
-    let nativeint ?suffix:_ i = Const_int (Nativeint.to_int i)
-    let float ?suffix:_ f = Const_float f
-    let char c = Const_char c
-    let string ?quotation_delimiter s = Const_string (s, quotation_delimiter)
-  end
-#endif
-
 module Arg = struct
-  type 'a conv = expression -> ('a, string) Result.result
+  type 'a conv = expression -> ('a, string) result
 
-  open Result
   let expr expr = Ok expr
 
   let int expr =
     match expr with
-#if OCAML_VERSION < (4, 03, 0)
-    | { pexp_desc = Pexp_constant (Const_int n) } -> Ok n
-#else
     | { pexp_desc = Pexp_constant (Pconst_integer (sn, _)) } -> Ok (int_of_string sn)
-#endif
     | _ -> Error "integer"
 
   let bool expr =
@@ -204,7 +248,9 @@ module Arg = struct
     | _ -> Error "boolean"
 
   let string expr =
-    Option.to_result ~none:"string" (string_of_expression_opt expr)
+    match expr with
+    | { pexp_desc = Pexp_constant (Pconst_string (n, _, None)) } -> Ok n
+    | _ -> Error "string"
 
   let char = function
     | { pexp_desc = Pexp_constant (Pconst_char c) } -> Ok c
@@ -231,21 +277,21 @@ module Arg = struct
   let get_attr ~deriver conv attr =
     match attr with
     | None -> None
-    | Some (Attribute_patt(loc, name,
-                           PStr [{ pstr_desc = Pstr_eval (expr, []) }])) ->
+    | Some { attr_name = {txt = name; loc = _};
+      attr_payload = PStr [{ pstr_desc = Pstr_eval (expr, []) }]; attr_loc = _ } ->
       begin match conv expr with
       | Ok v -> Some v
       | Error desc ->
         raise_errorf ~loc:expr.pexp_loc "%s: invalid [@%s]: %s expected" deriver name desc
       end
-    | Some (Attribute_patt(loc, name, _)) ->
+    | Some { attr_name = {txt = name; loc}; attr_payload = _; attr_loc = _ } ->
       raise_errorf ~loc "%s: invalid [@%s]: value expected" deriver name
 
   let get_flag ~deriver attr =
     match attr with
     | None -> false
-    | Some (Attribute_patt(_loc, name, PStr [])) -> true
-    | Some (Attribute_patt(loc, name, _)) ->
+    | Some { attr_name = _; attr_payload = PStr []; attr_loc = _ } -> true
+    | Some { attr_name = {txt = name; loc}; attr_payload = _; attr_loc = _ } ->
       raise_errorf ~loc "%s: invalid [@%s]: empty structure expected" deriver name
 
   let get_expr ~deriver conv expr =
@@ -257,36 +303,29 @@ end
 let attr_warning expr =
   let loc = !default_loc in
   let structure = {pstr_desc = Pstr_eval (expr, []); pstr_loc = loc} in
-  Attribute_expr(loc, "ocaml.warning", PStr [structure])
+  { attr_name = { txt = "ocaml.warning"; loc; };
+    attr_payload = PStr [structure];
+    attr_loc = loc;
+  }
 
-type quoter = {
-  mutable next_id : int;
-  mutable bindings : value_binding list;
-}
+type quoter = Expansion_helpers.Quoter.t
 
-let create_quoter () = { next_id = 0; bindings = [] }
+let create_quoter () = Expansion_helpers.Quoter.create ()
 
 let quote ~quoter expr =
-  let name = "__" ^ string_of_int quoter.next_id in
-  quoter.bindings <- (Vb.mk (pvar name) [%expr fun () -> [%e expr]]) :: quoter.bindings;
-  quoter.next_id <- quoter.next_id + 1;
-  [%expr [%e evar name] ()]
+  Expansion_helpers.Quoter.quote quoter expr
 
 let sanitize ?(module_=Lident "Ppx_deriving_runtime") ?(quoter=create_quoter ()) expr =
+  let loc = !Ast_helper.default_loc in
   let body =
-    let loc = !Ast_helper.default_loc in
     let attrs = [attr_warning [%expr "-A"]] in
     let modname = { txt = module_; loc } in
     Exp.open_ ~loc ~attrs
-#if OCAML_VERSION < (4, 08, 0)
-      Override modname
-#else
       (Opn.mk ~loc ~attrs ~override:Override (Mod.ident ~loc ~attrs modname))
-#endif
       expr in
-  match quoter.bindings with
-  | [] -> body
-  | bindings -> Exp.let_ Nonrecursive bindings body
+  let sanitized = Expansion_helpers.Quoter.sanitize quoter body in
+  (* ppxlib quoter uses Recursive, ppx_deriving's used Nonrecursive - silence warning *)
+  { sanitized with pexp_attributes = attr_warning [%expr "-39"] :: sanitized.pexp_attributes}
 
 let with_quoter fn a =
   let quoter = create_quoter () in
@@ -300,7 +339,7 @@ let path_of_type_decl ~path type_decl =
   | Some { ptyp_desc = Ptyp_constr ({ txt = lid }, _) } ->
     begin match lid with
     | Lident _ -> []
-    | Ldot (lid, _) -> Longident.flatten lid
+    | Ldot (lid, _) -> Ocaml_common.Longident.flatten lid
     | Lapply _ -> assert false
     end
   | _ -> path
@@ -327,8 +366,8 @@ let attr ~deriver name attrs =
     String.length str >= String.length prefix &&
       String.sub str 0 (String.length prefix) = prefix
   in
-  let attr_starts prefix (Attribute_patt(_loc, txt, _)) = starts prefix txt in
-  let attr_is name (Attribute_patt(_loc, txt, _)) = name = txt in
+  let attr_starts prefix attr = starts prefix attr.attr_name.txt in
+  let attr_is name attr = name = attr.attr_name.txt in
   let try_prefix prefix f =
     if List.exists (attr_starts prefix) attrs
     then prefix ^ name
@@ -356,29 +395,29 @@ let rec remove_pervasive_lid = function
 let remove_pervasives ~deriver typ =
   if attr_nobuiltin ~deriver typ.ptyp_attributes then typ
   else
-    let open Ast_mapper in
-    let map_typ mapper typ = match typ.ptyp_desc with
-      | Ptyp_constr (lid, l) ->
-        let lid = {lid with txt = remove_pervasive_lid lid.txt} in
-        {typ with
-         ptyp_desc = Ptyp_constr (lid, List.map (mapper.typ mapper) l)}
-      | Ptyp_class (lid, l) ->
-        let lid = {lid with txt = remove_pervasive_lid lid.txt} in
-        {typ with
-         ptyp_desc = Ptyp_class (lid, List.map (mapper.typ mapper) l)}
-      | _ -> default_mapper.typ mapper typ
-    in
-    let m = { default_mapper with typ = map_typ} in
-    m.typ m typ
+    let mapper = object
+        inherit Ppxlib.Ast_traverse.map as super
+
+        method! core_type typ =
+          match super#core_type typ with
+          | { ptyp_desc = Ptyp_constr (lid, l)} ->
+              let lid = {lid with txt = remove_pervasive_lid lid.txt} in
+              {typ with ptyp_desc = Ptyp_constr (lid, l)}
+          | { ptyp_desc = Ptyp_class (lid, l)} ->
+              let lid = {lid with txt = remove_pervasive_lid lid.txt} in
+              {typ with ptyp_desc = Ptyp_class (lid, l)}
+          | typ -> typ
+    end in
+    mapper#core_type typ
+
+let mkloc = Ocaml_common.Location.mkloc
 
 let fold_left_type_params fn accum params =
   List.fold_left (fun accum (param, _) ->
       match param with
       | { ptyp_desc = Ptyp_any } -> accum
       | { ptyp_desc = Ptyp_var name } ->
-#if OCAML_VERSION >= (4, 05, 0)
         let name = mkloc name param.ptyp_loc in
-#endif
         fn accum name
       | _ -> assert false)
     accum params
@@ -394,9 +433,7 @@ let fold_right_type_params fn params accum =
       match param with
       | { ptyp_desc = Ptyp_any } -> accum
       | { ptyp_desc = Ptyp_var name } ->
-#if OCAML_VERSION >= (4, 05, 0)
         let name = mkloc name param.ptyp_loc in
-#endif
         fn name accum
       | _ -> assert false)
     params accum
@@ -412,27 +449,19 @@ let free_vars_in_core_type typ =
     match typ with
     | { ptyp_desc = Ptyp_any } -> []
     | { ptyp_desc = Ptyp_var name } ->
-#if OCAML_VERSION >= (4, 05, 0)
       [mkloc name typ.ptyp_loc]
-#else
-      [name]
-#endif
     | { ptyp_desc = Ptyp_arrow (_, x, y) } -> free_in x @ free_in y
     | { ptyp_desc = (Ptyp_tuple xs | Ptyp_constr (_, xs)) } ->
       List.map free_in xs |> List.concat
     | { ptyp_desc = Ptyp_alias (x, name) } ->
-#if OCAML_VERSION >= (4, 05, 0)
       [mkloc name typ.ptyp_loc]
-#else
-      [name]
-#endif
       @ free_in x
     | { ptyp_desc = Ptyp_poly (bound, x) } ->
       List.filter (fun y -> not (List.mem y bound)) (free_in x)
     | { ptyp_desc = Ptyp_variant (rows, _, _) } ->
       List.map (
-          function Rtag_patt(_,_,ts) -> List.map free_in ts
-                 | Rinherit_patt(t) -> [free_in t]
+          function { prf_desc = Rtag(_,_,ts) } -> List.map free_in ts
+                 | { prf_desc = Rinherit(t) } -> [free_in t]
         ) rows |> List.concat |> List.concat
     | _ -> assert false
   in
@@ -440,11 +469,7 @@ let free_vars_in_core_type typ =
     let module StringSet = Set.Make(String) in
     let add (rev_names, txts) name =
       let txt =
-#if OCAML_VERSION >= (4, 05, 0)
         name.txt
-#else
-        name
-#endif
       in
       if StringSet.mem txt txts
       then (rev_names, txts)
@@ -468,47 +493,33 @@ let fresh_var bound =
 
 let poly_fun_of_type_decl type_decl expr =
   fold_right_type_decl (fun name expr ->
-#if OCAML_VERSION >= (4, 05, 0)
     let name = name.txt in
-#endif
     Exp.fun_ Label.nolabel None (pvar ("poly_"^name)) expr) type_decl expr
 
 let poly_fun_of_type_ext type_ext expr =
   fold_right_type_ext (fun name expr ->
-#if OCAML_VERSION >= (4, 05, 0)
     let name = name.txt in
-#endif
     Exp.fun_ Label.nolabel None (pvar ("poly_"^name)) expr) type_ext expr
 
 let poly_apply_of_type_decl type_decl expr =
   fold_left_type_decl (fun expr name ->
-#if OCAML_VERSION >= (4, 05, 0)
     let name = name.txt in
-#endif
     Exp.apply expr [Label.nolabel, evar ("poly_"^name)]) expr type_decl
 
 let poly_apply_of_type_ext type_ext expr =
   fold_left_type_ext (fun expr name ->
-#if OCAML_VERSION >= (4, 05, 0)
     let name = name.txt in
-#endif
     Exp.apply expr [Label.nolabel, evar ("poly_"^name)]) expr type_ext
 
 let poly_arrow_of_type_decl fn type_decl typ =
   fold_right_type_decl (fun name typ ->
-#if OCAML_VERSION >= (4, 05, 0)
     let name = name.txt in
-#endif
     Typ.arrow Label.nolabel (fn (Typ.var name)) typ) type_decl typ
 
 let poly_arrow_of_type_ext fn type_ext typ =
   fold_right_type_ext (fun name typ ->
     let var =
-#if OCAML_VERSION >= (4, 05, 0)
       Typ.var ~loc:name.loc name.txt
-#else
-      Typ.var name
-#endif
     in
     Typ.arrow Label.nolabel (fn var) typ) type_ext typ
 
@@ -545,11 +556,13 @@ let fold_exprs ?unit fn exprs =
     | None -> raise (Invalid_argument "Ppx_deriving.fold_exprs")
 
 let seq_reduce ?sep a b =
+  let loc = !Ast_helper.default_loc in
   match sep with
   | Some x -> [%expr [%e a]; [%e x]; [%e b]]
   | None -> [%expr [%e a]; [%e b]]
 
 let binop_reduce x a b =
+  let loc = !Ast_helper.default_loc in
   [%expr [%e x] [%e a] [%e b]]
 
 let strong_type_of_type ty =
@@ -583,13 +596,13 @@ let derive path pstr_loc item attributes fn arg =
           name,
           Options
             (options |> List.map (fun ({ txt }, expr) ->
-               String.concat "." (Longident.flatten txt), expr))
+               String.concat "." (Ocaml_common.Longident.flatten txt), expr))
         | { pexp_desc = Pexp_apply ({ pexp_desc = Pexp_ident name }, _) } ->
           name, Unknown_syntax
         | { pexp_loc } ->
           raise_errorf ~loc:pexp_loc "Unrecognized [@@deriving] syntax"
       in
-      let name, loc = String.concat "_" (Longident.flatten name.txt), name.loc in
+      let name, loc = String.concat "_" (Ocaml_common.Longident.flatten name.txt), name.loc in
       let is_optional, options =
         match options with
         | Unknown_syntax -> false, options
@@ -624,16 +637,12 @@ let derive_module_type_decl path module_type_decl pstr_loc item fn =
   derive path pstr_loc item attributes fn module_type_decl
 
 let module_from_input_name () =
-  match !Location.input_name with
+  match !Ocaml_common.Location.input_name with
   | ""
   | "//toplevel//" -> []
   | filename ->
     let capitalize =
-#if OCAML_VERSION >= (4, 03, 0)
       String.capitalize_ascii
-#else
-      String.capitalize
-#endif
     in
     match Filename.chop_suffix filename ".ml" with
       | exception _ ->
@@ -645,35 +654,25 @@ let module_from_input_name () =
 let pstr_desc_rec_flag pstr =
   match pstr with
   | Pstr_type(rec_flag, typ_decls) ->
-#if OCAML_VERSION < (4, 03, 0)
-    begin
-      if List.exists (fun ty -> has_attr "nonrec" ty.ptype_attributes) typ_decls then
-        Nonrecursive
-      else
-        Recursive
-    end
-#else
     rec_flag
-#endif
   | _ -> assert false
 
-let mapper =
-  let module_nesting = ref [] in
-  let with_module name f =
-    let old_nesting = !module_nesting in
-#if OCAML_VERSION >= (4, 10, 0)
-    begin match name with
-    | Some name -> module_nesting := !module_nesting @ [name]
-    | None -> ()
-    end;
-#else
-    module_nesting := !module_nesting @ [name];
-#endif
-    let result = f () in
-    module_nesting := old_nesting;
-    result
-  in
-  let expression mapper expr =
+let module_nesting = ref []
+
+let with_module name f =
+  let old_nesting = !module_nesting in
+  begin match name with
+  | Some name -> module_nesting := !module_nesting @ [name]
+  | None -> ()
+  end;
+  let result = f () in
+  module_nesting := old_nesting;
+  result
+
+class mapper = object (self)
+  inherit Ast_traverse.map as super
+
+  method! expression expr =
     match expr with
     | { pexp_desc = Pexp_extension ({ txt = name; loc }, payload) }
         when String.(length name >= 7 && sub name 0 7 = "derive.") ->
@@ -681,7 +680,9 @@ let mapper =
       let deriver =
         match lookup_internal_or_external name with
         | Some (Internal { core_type = Some deriver }) -> deriver
-        | Some _ -> raise_errorf ~loc "Deriver %s does not support inline notation" name
+        | Some _ ->
+            raise_errorf ~loc "Deriver %s does not support inline notation"
+              name
         | None -> raise_errorf ~loc "Cannot locate deriver %s" name
       in
       begin match payload with
@@ -691,12 +692,13 @@ let mapper =
     | { pexp_desc = Pexp_extension ({ txt = name; loc }, PTyp typ) } ->
       begin match lookup_internal_or_external name with
       | Some (Internal { core_type = Some deriver }) ->
-        Ast_helper.with_default_loc typ.ptyp_loc (fun () -> deriver typ)
-      | _ -> Ast_mapper.(default_mapper.expr) mapper expr
+        Ast_helper.with_default_loc typ.ptyp_loc (fun () ->
+          deriver typ)
+      | _ -> super#expression expr
       end
-    | _ -> Ast_mapper.(default_mapper.expr) mapper expr
-  in
-  let structure mapper items =
+    | _ -> super#expression expr
+
+  method! structure items =
     match items with
     | { pstr_desc = Pstr_type(_, typ_decls) as pstr_desc ; pstr_loc } :: rest when
         List.exists (fun ty -> has_attr "deriving" ty.ptype_attributes) typ_decls
@@ -708,90 +710,89 @@ let mapper =
         Ast_helper.with_default_loc pstr_loc (fun () ->
           derive_type_decl module_nesting typ_decls pstr_loc item
             (fun deriver -> deriver.type_decl_str))
-      in derived @ mapper.Ast_mapper.structure mapper rest
+      in derived @ self#structure rest
     | { pstr_desc = Pstr_typext typ_ext; pstr_loc } as item :: rest when
           has_attr "deriving" typ_ext.ptyext_attributes ->
       let derived =
         Ast_helper.with_default_loc pstr_loc (fun () ->
           derive_type_ext module_nesting typ_ext pstr_loc item
             (fun deriver -> deriver.type_ext_str))
-      in derived @ mapper.Ast_mapper.structure mapper rest
+      in derived @ self#structure rest
     | { pstr_desc = Pstr_modtype modtype; pstr_loc } as item :: rest when
           has_attr "deriving" modtype.pmtd_attributes ->
       let derived =
         Ast_helper.with_default_loc pstr_loc (fun () ->
-          derive_module_type_decl module_nesting modtype pstr_loc item
-            (fun deriver -> deriver.module_type_decl_str))
-      in derived @ mapper.Ast_mapper.structure mapper rest
+          derive_module_type_decl module_nesting modtype pstr_loc
+            item (fun deriver -> deriver.module_type_decl_str))
+      in derived @ self#structure rest
     | { pstr_desc = Pstr_module ({ pmb_name = { txt = name } } as mb) } as item :: rest ->
       let derived =
         { item with pstr_desc = Pstr_module (
             with_module name
-              (fun () -> mapper.Ast_mapper.module_binding mapper mb)) }
-      in derived :: mapper.Ast_mapper.structure mapper rest
+              (fun () -> self#module_binding mb)) }
+      in derived :: self#structure rest
     | { pstr_desc = Pstr_recmodule mbs } as item :: rest ->
       let derived =
         { item with pstr_desc = Pstr_recmodule (
             mbs |> List.map (fun ({ pmb_name = { txt = name } } as mb) ->
               with_module name
-                (fun () -> mapper.Ast_mapper.module_binding mapper mb))) }
-      in derived :: mapper.Ast_mapper.structure mapper rest
+                (fun () -> self#module_binding mb))) }
+      in derived :: self#structure rest
     | { pstr_loc } as item :: rest ->
-      let derived = mapper.Ast_mapper.structure_item mapper item
-      in derived :: mapper.Ast_mapper.structure mapper rest
+      let derived = self#structure_item item
+      in derived :: self#structure rest
     | [] -> []
-  in
-  let signature mapper items =
+
+  method! signature items =
     match items with
     | { psig_desc = Psig_type(_, typ_decls); psig_loc } as item :: rest when
-        List.exists (fun ty -> has_attr "deriving" ty.ptype_attributes) typ_decls ->
+        List.exists (fun ty -> has_attr "deriving" ty.ptype_attributes)
+          typ_decls ->
       let derived =
         Ast_helper.with_default_loc psig_loc (fun () ->
           derive_type_decl module_nesting typ_decls psig_loc item
             (fun deriver -> deriver.type_decl_sig))
-      in derived @ mapper.Ast_mapper.signature mapper rest
+      in derived @ self#signature rest
     | { psig_desc = Psig_typext typ_ext; psig_loc } as item :: rest when
         has_attr "deriving" typ_ext.ptyext_attributes ->
       let derived =
         Ast_helper.with_default_loc psig_loc (fun () ->
           derive_type_ext module_nesting typ_ext psig_loc item
-            (fun deriver -> deriver.type_ext_sig))
-      in derived @ mapper.Ast_mapper.signature mapper rest
+               (fun deriver -> deriver.type_ext_sig))
+      in derived @ self#signature rest
     | { psig_desc = Psig_modtype modtype; psig_loc } as item :: rest when
         has_attr "deriving" modtype.pmtd_attributes ->
       let derived =
         Ast_helper.with_default_loc psig_loc (fun () ->
           derive_module_type_decl module_nesting modtype psig_loc item
             (fun deriver -> deriver.module_type_decl_sig))
-      in derived @ mapper.Ast_mapper.signature mapper rest
+      in derived @ self#signature rest
     | { psig_desc = Psig_module ({ pmd_name = { txt = name } } as md) } as item :: rest ->
       let derived =
         { item with psig_desc = Psig_module (
             with_module name
-              (fun () -> mapper.Ast_mapper.module_declaration mapper md)) }
-      in derived :: mapper.Ast_mapper.signature mapper rest
+              (fun () -> self#module_declaration md)) }
+      in derived :: self#signature rest
     | { psig_desc = Psig_recmodule mds } as item :: rest ->
       let derived =
         { item with psig_desc = Psig_recmodule (
             mds |> List.map (fun ({ pmd_name = { txt = name } } as md) ->
               with_module name
-                (fun () -> mapper.Ast_mapper.module_declaration mapper md))) }
-      in derived :: mapper.Ast_mapper.signature mapper rest
+                (fun () -> self#module_declaration md))) }
+      in derived :: self#signature rest
     | { psig_loc } as item :: rest ->
-      let derived =
-        mapper.Ast_mapper.signature_item mapper item
-      in derived :: mapper.Ast_mapper.signature mapper rest
+      let derived = self#signature_item item
+      in derived :: self#signature rest
     | [] -> []
-  in
-  Ast_mapper.{default_mapper with
-    expr = expression;
-    structure = (fun mapper items ->
-      module_nesting := module_from_input_name ();
-      structure { mapper with structure; signature } items);
-    signature = (fun mapper items ->
-      module_nesting := module_from_input_name ();
-      signature { mapper with structure; signature } items)
-  }
+end
+
+let map_structure s =
+  module_nesting := module_from_input_name ();
+  (new mapper)#structure s
+
+let map_signature s =
+  module_nesting := module_from_input_name ();
+  (new mapper)#signature s
 
 let hash_variant s =
   let accu = ref 0 in
@@ -806,6 +807,6 @@ let hash_variant s =
 (* This is only used when ppx_deriving is linked as part of an ocaml-migrate-parsetre
    driver. *)
 let () =
-  Migrate_parsetree.Driver.register ~name:"ppx_deriving"
-    (module Migrate_parsetree.OCaml_current)
-    (fun _ _ -> mapper)
+  Ppxlib.Driver.register_transformation "ppx_deriving"
+    ~impl:map_structure
+    ~intf:map_signature
