@@ -12,8 +12,8 @@ type a7 = char       [@@deriving show]
 type a8 = string     [@@deriving show]
 type a9 = bytes      [@@deriving show]
 type r  = int ref    [@@deriving show]
-type r2 = int Pervasives.ref [@@ocaml.warning "-3"] [@@deriving show]
-type r3 = int Pervasives.ref ref [@@ocaml.warning "-3"] [@@deriving show]
+type r2 = int Stdlib.ref [@@deriving show]
+type r3 = int Stdlib.ref ref [@@deriving show]
 type l  = int list   [@@deriving show]
 type a  = int array  [@@deriving show]
 type o  = int option [@@deriving show]
@@ -47,16 +47,14 @@ let test_variant ctxt =
   assert_equal ~printer "(Test_deriving_show.Bar (1, \"foo\"))" (show_v (Bar (1, "foo")));
   assert_equal ~printer "(Test_deriving_show.Baz \"foo\")"      (show_v (Baz "foo"))
 
-#if OCAML_VERSION >= (4, 03, 0)
 type rv = RFoo | RBar of { x: int; y: string } | RBaz of { z: string } [@@deriving show]
-let test_variant_record ctxt =
+let test_rv_record ctxt =
   assert_equal ~printer "Test_deriving_show.RFoo"
                         (show_rv RFoo);
   assert_equal ~printer "Test_deriving_show.RBar {x = 1; y = \"foo\"}"
                         (show_rv (RBar {x=1; y="foo"}));
-  assert_equal ~printer "(Test_deriving_show.RBaz {z = \"foo\"}"
+  assert_equal ~printer "Test_deriving_show.RBaz {z = \"foo\"}"
                         (show_rv (RBaz {z="foo"}))
-#endif
 
 type vn = Foo of int option [@@deriving show]
 let test_variant_nest ctxt =
@@ -85,7 +83,6 @@ let test_record ctxt =
   assert_equal ~printer "{ Test_deriving_show.f1 = 1; f2 = \"foo\"; f3 = <opaque> }"
                         (show_re { f1 = 1; f2 = "foo"; f3 = 1.0 })
 
-#if OCAML_VERSION >= (4, 03, 0)
 type variant = Foo of {
   f1 : int;
   f2 : string;
@@ -95,7 +92,6 @@ let test_variant_record ctxt =
   assert_equal ~printer
     "Test_deriving_show.Foo {f1 = 1; f2 = \"foo\"; f3 = <opaque>}"
     (show_variant (Foo { f1 = 1; f2 = "foo"; f3 = 1.0 }))
-#endif
 
 
 module M : sig
@@ -166,7 +162,6 @@ let test_mrec ctxt =
   assert_equal ~printer "(Test_deriving_show.B\n   { Test_deriving_show.x = 12; r = (Test_deriving_show.F 16) })" (show_foo e1)
 
 
-#if OCAML_VERSION >= (4, 03, 0)
 type i_has_result = I_has of (bool, string) result [@@deriving show]
 
 let test_result ctxt =
@@ -176,14 +171,12 @@ let test_result ctxt =
     (show_i_has_result (I_has (Ok true)));
   assert_equal ~printer "(Test_deriving_show.I_has (Error \"err\"))"
     (show_i_has_result (I_has (Error "err")))
-#endif
 
-type i_has_result_result = I_has of (bool, string) Result.result [@@deriving show]
+type i_has_result_result = I_has of (bool, string) result [@@deriving show]
 
 let test_result_result ctxt =
-  let open Result in
   assert_equal ~printer "(Ok 100)"
-    ([%show: (int, bool) result] (Result.Ok 100));
+    ([%show: (int, bool) result] (Ok 100));
   assert_equal ~printer "(Test_deriving_show.I_has (Ok true))"
     (show_i_has_result_result (I_has (Ok true)));
   assert_equal ~printer "(Test_deriving_show.I_has (Error \"err\"))"
@@ -254,14 +247,13 @@ let test_paths_printer ctxt =
 let suite = "Test deriving(show)" >::: [
     "test_alias"           >:: test_alias;
     "test_variant"         >:: test_variant;
+    "test_rv_record"       >:: test_rv_record;
     "test_variant_nest"    >:: test_variant_nest;
     "test_tuple"           >:: test_tuple;
     "test_poly"            >:: test_poly;
     "test_poly_inherit"    >:: test_poly_inherit;
     "test_record"          >:: test_record;
-#if OCAML_VERSION >= (4, 03, 0)
     "test_variant_record"  >:: test_variant_record;
-#endif
     "test_abstr"           >:: test_abstr;
     "test_custom"          >:: test_custom;
     "test_parametric"      >:: test_parametric;
@@ -273,9 +265,7 @@ let suite = "Test deriving(show)" >::: [
     "test_poly_app"        >:: test_poly_app;
     "test_variant_printer" >:: test_variant_printer;
     "test_paths"           >:: test_paths_printer;
-#if OCAML_VERSION >= (4, 03, 0)
     "test_result"          >:: test_result;
-#endif
     "test_result_result"   >:: test_result_result;
   ]
 
